using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;


namespace StyledControls
{
	public class StyledButton : Button
	{
		public StyledButton() : base()
		{
			SetStyle(ControlStyles.ResizeRedraw, true);
			RebuildDrawData();
		}

//		private bool isPushed;
//		[DefaultValue(false)]
//		public bool Pushed
//		{
//			get { return isPushed; }
//			set
//			{
//				isPushed = value;
//				Invalidate();
//			}
//		}

		#region changes events

		protected override void OnResize(EventArgs e)
		{
			RebuildDrawData();

			base.OnResize(e);
		}

		protected override void OnMove(EventArgs e)
		{
			RebuildDrawData();

			base.OnMove(e);
		}

		protected override void OnBackColorChanged(EventArgs e)
		{
			base.OnBackColorChanged(e);

			lightColor = ControlPaint.LightLight(BackColor);
			lightLightColor = ControlPaint.LightLight(lightColor);
			darkColor = BackColor;
			
//			int mid = (lightColor.A + lightColor.R + lightColor.G + lightColor.B) / 4;
//			grayLightColor = Color.FromArgb(mid, mid, mid, mid);
//			mid = (darkColor.A + darkColor.R + darkColor.G + darkColor.B) / 4;
//			grayDarkColor = Color.FromArgb(mid, mid, mid, mid);
		}

		#endregion

		#region mouse events

		protected override void OnMouseEnter(EventArgs e)
		{
			hover = true;
			Invalidate();

			base.OnMouseEnter(e);
		}

		protected override void OnMouseLeave(EventArgs e)
		{
			hover = false;
			Invalidate();

			base.OnMouseLeave(e);
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			if(e.Button == MouseButtons.Left)
			{
				pushed = true;
				Invalidate();
			}

			base.OnMouseDown(e);
		}


		protected override void OnMouseUp(MouseEventArgs e)
		{
			if(e.Button == MouseButtons.Left)
			{
				pushed = false;
				Invalidate();
			}

			base.OnMouseUp(e);
		}

		#endregion

		protected GraphicsPath regionPath;
		protected GraphicsPath parentPath;
		protected GraphicsPath glowPath;
		protected Rectangle regionRect;
		protected Rectangle borderRect;
		protected Rectangle glowRect;
		protected Rectangle parentRect;
		protected Rectangle textRect;
//		protected Color grayLightColor;
//		protected Color grayDarkColor;
		protected Color lightColor;
		protected Color darkColor;
		protected Color pFrontColor;
		protected Color pBackColor;
		protected Color lightLightColor;
		protected bool hover = false;
		protected bool pushed = false;

		protected virtual void RebuildDrawData()
		{
			int len = Width - Height;
			regionPath = new GraphicsPath();
			regionPath.AddArc(0, 0, Height, Height, 90, 180);
			regionPath.AddArc(len, 0, Height, Height, -90, 180);
			regionPath.CloseFigure();

			Region = new Region(regionPath);
			
			regionRect = new Rectangle(0, -1, Width, Height + 2);
			borderRect = new Rectangle(-4, -4, Width + 8, Height + 8);

			int hh = (int)(Height / 1.6f);
			glowPath = new GraphicsPath();
			glowPath.AddArc(0, 0, Height, hh, 90, 180);
			glowPath.AddArc(len, 0, Height, hh, -90, 180);
			glowPath.CloseFigure();

			glowRect = new Rectangle(0, -1, Width, hh + 2);

			parentPath = new GraphicsPath();
			parentPath.AddArc(Left, Top, Height, Height, 90, 180);
			parentPath.AddArc(Left + len, Top, Height, Height, -90, 180);
			parentPath.CloseFigure();

			parentRect = new Rectangle(Left - 4, Top - 4, Width + 8, Height + 8);
			
			textRect = ClientRectangle;
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.HighQuality;


//			if(Enabled)
//			{
				pFrontColor = hover ? lightLightColor : lightColor;
				pBackColor = darkColor;
//			}
//			else
//			{
//				pFrontColor = grayLightColor;
//				pBackColor = grayDarkColor;
//			}

			DrawButton(g);

			if(Enabled)
			{
				if(pushed)
					DrawPushed(g);
				else if(Focused)
					DrawFocused(g);
			}

			using(LinearGradientBrush gb = new LinearGradientBrush(borderRect, pFrontColor, pBackColor, LinearGradientMode.Vertical))
			using(Pen extPen = new Pen(gb, 4))
			{
				extPen.Alignment = PenAlignment.Center;
				g.DrawPath(extPen, regionPath);
			}

			if(Parent != null)
			{
				using(LinearGradientBrush gb = new LinearGradientBrush(parentRect, pFrontColor, pBackColor, LinearGradientMode.Vertical))
					using(Pen p = new Pen(gb, 4))
					{
						p.Alignment = PenAlignment.Center;
						Graphics pg = Parent.CreateGraphics();
						pg.SmoothingMode = SmoothingMode.HighQuality;
						pg.DrawPath(p, parentPath);
					}
			}

			DrawText(g);
		}

		protected virtual void DrawButton(Graphics g)
		{
			using(SolidBrush sb = new SolidBrush(pBackColor))
				g.FillPath(sb, regionPath);

			using(LinearGradientBrush gb = new LinearGradientBrush(glowRect, pFrontColor, pBackColor, LinearGradientMode.Vertical))
				g.FillPath(gb, glowPath);
		}

		protected virtual void DrawFocused(Graphics g)
		{
			using(Pen intPen = new Pen(pFrontColor, 4))
			{
				intPen.Alignment = PenAlignment.Inset;
				g.DrawPath(intPen, regionPath);
			}
		}

		protected virtual void DrawPushed(Graphics g)
		{
			using(LinearGradientBrush gb = new LinearGradientBrush(ClientRectangle, pBackColor, pFrontColor, LinearGradientMode.Vertical))
			using(Pen intPen = new Pen(gb, 4))
			{
				intPen.Alignment = PenAlignment.Inset;
				g.DrawPath(intPen, regionPath);
			}
		}

		protected virtual void DrawText(Graphics g)
		{
			StringFormat sf = new StringFormat();
			sf.Alignment = sf.LineAlignment = StringAlignment.Center;
			sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show;

			if(Enabled)
			{
				using(SolidBrush sb = new SolidBrush(ForeColor))
					g.DrawString(Text, Font, sb, textRect, sf);
			}
			else
			{
				ControlPaint.DrawStringDisabled(g, Text, Font, ForeColor, textRect, sf);
			}
		}

	}
}
