using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace StyledControls
{
	public class StyledCheckBox : StyledButton
	{
		public StyledCheckBox() : base() {}

		private bool isChecked = false;
		[DefaultValue(false)]
		public bool Checked
		{
			get { return isChecked; }
			set 
			{
				isChecked = value; 

				Invalidate(checkRect);
			}
		}

		private void SetRadio(bool val)
		{
			if(val)
			{
				if(Parent != null)
					foreach(StyledCheckBox cb in Parent.Controls)
						if(cb.Radio && cb.Checked)
							cb.Checked = false;

				isChecked = val;
				Invalidate(checkRect);
			}
		}

		private bool radio = false;
		[DefaultValue(false)]
		public bool Radio
		{
			get { return radio; }
			set 
			{
				radio = value; 
				Invalidate();
			}
		}

		private bool radioPush = true;
		[DefaultValue(true)]
		public bool RadioPush
		{
			get { return radioPush; }
			set { radioPush = value; }
		}

		protected override void OnClick(EventArgs e)
		{
			if(radio)
				SetRadio(!Checked);
			else
				Checked = !Checked;

			base.OnClick(e);
		}

		protected Rectangle checkRect;
		protected Rectangle checkMarkRect;

		protected override void RebuildDrawData()
		{
			base.RebuildDrawData();

			textRect = new Rectangle(Height, 0, Width - Height, Height);
			checkRect = new Rectangle(10, 6, Height - 12, Height - 12);
			checkMarkRect = new Rectangle(12, 8, Height - 16, Height - 16);
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint(e);

			DrawBox(e.Graphics);

//			ButtonState state = isChecked ? ButtonState.Checked | ButtonState.Flat : ButtonState.Flat;
//			if(Radio)
//				ControlPaint.DrawRadioButton(e.Graphics, checkRect, state);
//			else
//				ControlPaint.DrawCheckBox(e.Graphics, checkRect, state);
		}

		protected override void DrawPushed(Graphics g)
		{
			if(!radio || (!radioPush && !isChecked))
				base.DrawPushed(g);
		}

		protected virtual void DrawBox(Graphics g)
		{
			using(Pen p = new Pen(ForeColor, 2))
			using(SolidBrush b = new SolidBrush(ForeColor))
			{
//				p.Alignment = PenAlignment.Inset;

				if(Radio)
				{
					g.DrawEllipse(p, checkRect);
					if(Checked)
						g.FillEllipse(b, checkMarkRect);
				}
				else
				{
					g.DrawRectangle(p, checkRect);
					if(Checked)
						g.FillRectangle(b, checkMarkRect);
				}
			}
		}

	}

}

