using System;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace StyledControls
{

	public class StyledForm : System.Windows.Forms.Form
	{
		private System.ComponentModel.Container components = null;

		public StyledForm()
		{
			SetStyle(ControlStyles.ResizeRedraw, true);
			RebuildDrawData();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		protected override void OnResize(EventArgs e)
		{
			RebuildDrawData();

			base.OnResize(e);
		}

		protected override void OnBackColorChanged(EventArgs e)
		{
			base.OnBackColorChanged(e);

			lightColor = ControlPaint.LightLight(BackColor);
			lightLightColor = ControlPaint.LightLight(lightColor);
		}

//		protected override void OnMouseEnter(EventArgs e)
//		{
//			hover = true;
//			Invalidate();
//
//			base.OnMouseEnter(e);
//		}
//
//		protected override void OnMouseLeave(EventArgs e)
//		{
//			hover = false;
//			Invalidate();
//
//			base.OnMouseLeave(e);
//		}
//
//		protected override void OnMouseDown(MouseEventArgs e)
//		{
//			pushed = true;
//			Invalidate();
//
//			base.OnMouseDown(e);
//		}
//
//		protected override void OnMouseUp(MouseEventArgs e)
//		{
//			pushed = false;
//			Invalidate();
//
//			base.OnMouseUp(e);
//		}

		private GraphicsPath regionPath;
//		private GraphicsPath glowPath;
//		private Rectangle glowRect;
		private Color lightColor;
		private Color lightLightColor;

		private void RebuildDrawData()
		{
			int rad = SystemInformation.CaptionHeight;
			int hrad = rad / 2;
			int len = Width - rad;


			regionPath = new GraphicsPath();
			regionPath.AddArc(0, 0, rad, rad, 180, 90);
			regionPath.AddLine(rad, 0, Width, 0);
			regionPath.AddLine(Width, 0, Height - hrad, 0);
			regionPath.AddArc(Width - rad, Height - rad, rad, rad, 0, 90);
			regionPath.AddLine(Width - hrad, Height, hrad, Height);
			regionPath.AddArc(0, Height - rad, rad, rad, 90, 90);
			regionPath.AddLine(0, Height - hrad, 0, hrad);

			Region = new Region(regionPath);

			//			int hh = (int)(Height / 1.6f);
			//			glowPath = new GraphicsPath();
			//			glowPath.AddArc(0, 0, Height, hh, 90, 180);
			//			glowPath.AddLine(Height, 0, len, 0);
			//			glowPath.AddArc(len, 0, Height, hh, -90, 180);
			//			glowPath.AddLine(len, hh, Height, hh);
			//
			//			glowRect = new Rectangle(0, 0, Width, hh + 1);
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.HighQuality;

			using(SolidBrush sb = new SolidBrush(BackColor))
				g.FillPath(sb, regionPath);

			Color glowColor = Focused ? lightLightColor : lightColor;

//			using(LinearGradientBrush gb = new LinearGradientBrush(glowRect, glowColor, BackColor, LinearGradientMode.Vertical))
//				g.FillPath(gb, glowPath);


			using(Pen p = new Pen(glowColor, 4))
				g.DrawPath(p, regionPath);

//			StringFormat sf = new StringFormat();
//			sf.Alignment = sf.LineAlignment = StringAlignment.Center;
//			using(SolidBrush sb = new SolidBrush(ForeColor))
//				g.DrawString(Text, Font, sb, ClientRectangle, sf);
		}

	}
}
