using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace StyledControls
{
	public class StyledPanel : Panel
	{
		public StyledPanel() : base()
		{
			SetStyle(ControlStyles.ResizeRedraw, true);
			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);
			RebuildDrawData();
		}

		#region controls

		protected override void OnControlAdded(ControlEventArgs e)
		{
			e.Control.GotFocus += new EventHandler(Control_GotFocus);
			e.Control.LostFocus += new EventHandler(Control_LostFocus);
			base.OnControlAdded(e);
		}

		protected override void OnControlRemoved(ControlEventArgs e)
		{
			e.Control.GotFocus -= new EventHandler(Control_GotFocus);
			e.Control.LostFocus -= new EventHandler(Control_LostFocus);
			base.OnControlRemoved(e);
		}

		private StyledPanelHeader header;
		public StyledPanelHeader Header
		{
			get { return header; }
			set
			{
				if(value != null)
				{
					header = value;
					header.Click += new EventHandler(Header_Click);
					header.GotFocus += new EventHandler(Control_GotFocus);
					header.LostFocus += new EventHandler(Control_LostFocus);
				}
			}
		}
		bool ShouldSerializeHeader(){ return header != null; }
		void ResetHeader() { header = null; }

		private StyledPanelFooter footer;
		public StyledPanelFooter Footer
		{
			get { return footer; }
			set
			{
				if(value != null)
				{
					footer = value;
					footer.Click += new EventHandler(Header_Click);
					footer.GotFocus += new EventHandler(Control_GotFocus);
					footer.LostFocus += new EventHandler(Control_LostFocus);
				}
			}
		}
		bool ShouldSerializeFooter(){ return footer != null; }
		void ResetFooter() { footer = null; }

		private void Header_Click(object sender, EventArgs e)
		{
			OnGotFocus(e);

			if(Controls[0] != null)
				Controls[0].Focus();
		}

		private void Control_GotFocus(object sender, EventArgs e)
		{
			OnGotFocus(e);
		}

		private void Control_LostFocus(object sender, EventArgs e)
		{
			if(!ContainsFocus)
				OnLostFocus(e);
		}

		#endregion

		#region changed events

		protected override void OnResize(EventArgs e)
		{
			RebuildDrawData();

			base.OnResize(e);
		}

		protected override void OnMove(EventArgs e)
		{
			RebuildDrawData();

			base.OnMove(e);
		}

		protected override void OnBackColorChanged(EventArgs e)
		{
			base.OnBackColorChanged(e);

			lightColor = ControlPaint.LightLight(BackColor);
			lightLightColor = ControlPaint.LightLight(lightColor);
			darkColor = BackColor;
		}

		#endregion

		#region focus

		protected override void OnClick(EventArgs e)
		{
			OnGotFocus(e);

			if(Controls[0] != null)
				Controls[0].Focus();
		}

		protected override void OnGotFocus(EventArgs e)
		{
			if(Header != null)
				Header.Active = true;
			if(Footer != null)
				Footer.Active = true;

			focused = true;
			Invalidate();
		}

		protected override void OnLostFocus(EventArgs e)
		{
			if(Header != null)
				Header.Active = false;
			if(Footer != null)
				Footer.Active = false;

			focused = false;
			Invalidate();
		}

		#endregion

		private Rectangle fillRect;
//		private Rectangle parentRect;
//		private Rectangle parentBorderRect;
		private Color lightColor;
		private Color darkColor;
		private Color lightLightColor;
		private bool focused;

		private void RebuildDrawData()
		{
			fillRect = new Rectangle(0, 0, Width, Height / 2);
//			parentRect = new Rectangle(Left, Top, Width, Height);
//			parentBorderRect = new Rectangle(Left, Top - 2, Width, Height / 2 + 2);
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.HighQuality;

			Color glowColor = focused ? lightLightColor : lightColor;

//			if(Parent != null)
//			{
//				using(LinearGradientBrush gb = new LinearGradientBrush(parentBorderRect, darkColor, glowColor, LinearGradientMode.Vertical))
//				{	
//					gb.WrapMode = WrapMode.TileFlipXY;
//					using(Pen p = new Pen(darkColor, 4))
//					{
//						p.Alignment = PenAlignment.Center;
//						Graphics pg = Parent.CreateGraphics();
//						pg.SmoothingMode = SmoothingMode.HighQuality;
//						pg.DrawRectangle(p, parentRect);
//					}
//				}
//			}

			using(LinearGradientBrush gb = new LinearGradientBrush(fillRect, darkColor, glowColor, LinearGradientMode.Vertical))
			{
				gb.WrapMode = WrapMode.TileFlipXY;
				g.FillRectangle(gb, ClientRectangle);
			}

			using(HatchBrush hb = new HatchBrush(HatchStyle.LargeGrid, Color.FromArgb(0x11, glowColor), Color.Transparent))
				g.FillRectangle(hb, ClientRectangle);
		}

	}
}