using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;

namespace StyledControls
{
	public class StyledPanelHeader : Control
	{
		public StyledPanelHeader() : base()
		{
			SetStyle(ControlStyles.ResizeRedraw, true);
			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);
			SetStyle(ControlStyles.Selectable, false);
			Size = new Size(112, 32);
			RebuildDrawData();
		}

		#region changed events

		protected override void OnResize(EventArgs e)
		{
			RebuildDrawData();

			base.OnResize(e);
		}

		protected override void OnMove(EventArgs e)
		{
			RebuildDrawData();

			base.OnMove(e);
		}

		protected override void OnBackColorChanged(EventArgs e)
		{
			lightColor = ControlPaint.LightLight(BackColor);
			lightLightColor = ControlPaint.LightLight(lightColor);
			darkColor = BackColor;

			base.OnBackColorChanged(e);
		}

		#endregion

		private GraphicsPath regionPath;
		private GraphicsPath parentPath;
		private GraphicsPath glowPath;
		private Rectangle regionRect;
		private Rectangle borderRect;
		private Rectangle glowRect;
		private Rectangle parentRect;
		private Rectangle textRect;
		private Color lightColor;
		private Color darkColor;
		private Color lightLightColor;

		private bool focused = false;
		[DefaultValue(false)]
		public bool Active
		{
			get { return focused; }
			set 
			{
				focused = value; 
				Invalidate();
			}
		}

		private void RebuildDrawData()
		{
			int len = Width - Height;
			int dh = Height * 2;
			regionPath = new GraphicsPath();
			regionPath.AddArc(0, 0, Height, dh, 180, 90);
			regionPath.AddArc(len, 0, Height, dh, -90, 90);
			regionPath.CloseFigure();

			Region = new Region(regionPath);
			
			regionRect = new Rectangle(0, -1, Width, Height + 2);
			borderRect = new Rectangle(-4, -4, Width + 8, Height + 8);

			int hh = (int)(Height / 1.6f);
			glowPath = new GraphicsPath();
			glowPath.AddArc(0, 0, Height, hh, 90, 180);
			glowPath.AddArc(len, 0, Height, hh, -90, 180);
			glowPath.CloseFigure();

			glowRect = new Rectangle(0, -1, Width, hh + 2);

			parentPath = new GraphicsPath();
			parentPath.AddArc(Left, Top, Height, dh, 180, 90);
			parentPath.AddArc(Left + len, Top, Height, dh, -90, 90);
			parentPath.CloseFigure();

			parentRect = new Rectangle(Left - 4, Top - 4, Width + 8, Height + 8);
		
			textRect = ClientRectangle;
			textRect.X += 8;
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.HighQuality;

			using(SolidBrush sb = new SolidBrush(darkColor))
				g.FillPath(sb, regionPath);

			Color glowColor = focused ? lightLightColor : lightColor;

			using(LinearGradientBrush gb = new LinearGradientBrush(glowRect, glowColor, darkColor, LinearGradientMode.Vertical))
				g.FillPath(gb, glowPath);

			using(LinearGradientBrush gb = new LinearGradientBrush(borderRect, glowColor, darkColor, LinearGradientMode.Vertical))
			using(Pen extPen = new Pen(gb, 4))
			{
				extPen.Alignment = PenAlignment.Center;
				g.DrawPath(extPen, regionPath);
			}

			if(Parent != null)
			{
				using(LinearGradientBrush gb = new LinearGradientBrush(parentRect, glowColor, darkColor, LinearGradientMode.Vertical))
				using(Pen p = new Pen(gb, 4))
				{
					p.Alignment = PenAlignment.Center;
					Graphics pg = Parent.CreateGraphics();
					pg.SmoothingMode = SmoothingMode.HighQuality;
					pg.DrawPath(p, parentPath);
				}
			}


			StringFormat sf = new StringFormat();
			sf.Alignment = StringAlignment.Near;
			sf.LineAlignment = StringAlignment.Center;
			using(SolidBrush sb = new SolidBrush(ForeColor))
				g.DrawString(Text, Font, sb, textRect, sf);
		}

	}
}
