using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;


namespace StyledControls
{
	public class StyledTextBox : TextBox
	{
		public StyledTextBox() : base()
		{
			SetStyle(ControlStyles.ResizeRedraw, true);
			BorderStyle = BorderStyle.None;
//			RebuildDrawData();
		}

		#region changes events

		protected override void OnGotFocus(EventArgs e)
		{
			base.OnGotFocus(e);

			if(Parent != null)
				Parent.Invalidate(parentRect);
		}

		protected override void OnLostFocus(EventArgs e)
		{
			base.OnLostFocus(e);

			if(Parent != null)
				Parent.Invalidate(parentRect);
		}

		protected override void OnParentBackColorChanged(EventArgs e)
		{
			base.OnParentBackColorChanged(e);

			if(Parent != null)
			{
				BackColor = Parent.BackColor;
//				ForeColor = Parent.ForeColor;
			}
		}

		protected override void OnParentForeColorChanged(EventArgs e)
		{
			base.OnParentForeColorChanged(e);

			if(Parent != null)
			{
//				BackColor = Parent.BackColor;
				ForeColor = Parent.ForeColor;
			}
		}

		protected override void OnParentChanged(EventArgs e)
		{
			base.OnParentChanged(e);

			if(Parent != null)
			{
				RebuildDrawData();
				Parent.Paint += new PaintEventHandler(Parent_Paint);
			}
//			else
//			{
////				Parent.Paint -= new PaintEventHandler(Parent_Paint);
//			}
		}

		protected override void OnResize(EventArgs e)
		{
			if(Parent != null)
				RebuildDrawData();

			base.OnResize(e);
		}

		protected override void OnMove(EventArgs e)
		{
			if(Parent != null)
				RebuildDrawData();

			base.OnMove(e);
		}

		protected override void OnBackColorChanged(EventArgs e)
		{
			base.OnBackColorChanged(e);

			lightColor = ControlPaint.LightLight(BackColor);
			darkColor = BackColor;
			
			int mid = (lightColor.A + lightColor.R + lightColor.G + lightColor.B) / 4;
			grayLightColor = Color.FromArgb(mid, mid, mid, mid);
			mid = (darkColor.A + darkColor.R + darkColor.G + darkColor.B) / 4;
			grayDarkColor = Color.FromArgb(mid, mid, mid, mid);
		}

		#endregion

		protected GraphicsPath regionPath;
		protected GraphicsPath parentPath;
		protected GraphicsPath glowPath;
		protected Rectangle regionRect;
		protected Rectangle borderRect;
		protected Rectangle glowRect;
		protected Rectangle parentRect;
		protected Color grayLightColor;
		protected Color grayDarkColor;
		protected Color lightColor;
		protected Color darkColor;
		protected Color pFrontColor;
		protected Color pBackColor;

		private void RebuildDrawData()
		{
			int x = Left - 10;
			int y = Top - 6;
			int w = Width + 16;
			int h = Height + 12;
			int len = x + w - h;

			regionPath = new GraphicsPath();
			regionPath.AddArc(x, y, h, h, 90, 180);
			regionPath.AddArc(len, y, h, h, -90, 180);
			regionPath.CloseFigure();

//			Region = new Region(regionPath);
			
			regionRect = new Rectangle(x, y, w, h);
			borderRect = new Rectangle(x - 4, y - 4, w + 8, h + 8);

//			int hh = (int)(h / 1.6f);
//			glowPath = new GraphicsPath();
//			glowPath.AddArc(x, y, h, hh, 90, 180);
//			glowPath.AddArc(len, y, h, hh, -90, 180);
//			glowPath.CloseFigure();

//			glowRect = new Rectangle(x, y - 1, w, hh + 2);

			parentPath = new GraphicsPath();
			parentPath.AddArc(x, y, h, h, 90, 180);
			parentPath.AddArc(len, y, h, h, -90, 180);
			parentPath.CloseFigure();

			parentRect = new Rectangle(x - 4, y - 4, w + 8, h + 8);
		}

//		protected override void OnPaint(PaintEventArgs e)
		private void Parent_Paint(object sender, PaintEventArgs e)
		{
			//			if(Parent != null)
			//			{
			Graphics g = e.Graphics;Parent.CreateGraphics();
			g.SmoothingMode = SmoothingMode.HighQuality;

			pFrontColor = lightColor;
			pBackColor = darkColor;

			DrawLabel(g);


			if(Enabled && Focused)
				DrawFocused(g);
			else
				using(LinearGradientBrush gb = new LinearGradientBrush(parentRect, pBackColor, pFrontColor, LinearGradientMode.Vertical))
				using(Pen intPen = new Pen(gb, 4))
				{
					intPen.Alignment = PenAlignment.Inset;
					g.DrawPath(intPen, regionPath);
				}

			using(LinearGradientBrush gb = new LinearGradientBrush(borderRect, pFrontColor, pBackColor, LinearGradientMode.Vertical))
			using(Pen extPen = new Pen(gb, 4))
			{
				extPen.Alignment = PenAlignment.Center;
				g.DrawPath(extPen, regionPath);
			}

			using(LinearGradientBrush gb = new LinearGradientBrush(parentRect, pFrontColor, pBackColor, LinearGradientMode.Vertical))
			using(Pen p = new Pen(gb, 4))
			{
				p.Alignment = PenAlignment.Center;
				g.SmoothingMode = SmoothingMode.HighQuality;
				g.DrawPath(p, parentPath);
			}
			//			}

		}

		protected virtual void DrawLabel(Graphics g)
		{
			using(SolidBrush sb = new SolidBrush(pBackColor))
				g.FillPath(sb, regionPath);

//			using(LinearGradientBrush gb = new LinearGradientBrush(glowRect, pFrontColor, pBackColor, LinearGradientMode.Vertical))
//				g.FillPath(gb, glowPath);
		}

		protected virtual void DrawFocused(Graphics g)
		{
			using(Pen intPen = new Pen(pFrontColor, 4))
			{
				intPen.Alignment = PenAlignment.Inset;
				g.DrawPath(intPen, regionPath);
			}
		}
	}
}
